/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package net.cwroethel.swt.popupcalendar;

import java.util.Locale;

import org.eclipse.swt.widgets.Control;

/*******************************************************************************
 * Copyright (c) 2005 Will Roethel. All rights reserved. This program and the
 * accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html Contributors: Will Roethel -
 * initial API and implementation
 ******************************************************************************/

/**
 * <p>
 * Class containing all style information on the GUI elements used by
 * DateChooser. The preferred way to override these settings is to use the
 * methods provided by PopupCalendar to modify the look and feel of the swing
 * components.
 * 
 * @author Will Roethel, http://www.cwroethel.net
 * @version $Revision: 1.2 $
 * @see DateChooser
 * @see DateChooserPanel
 */
public abstract class DateChooserStyle {
    static int       daySize = 18;

    protected Locale locale  = Locale.getDefault();


    public DateChooserStyle() {
    }


    /**
     * Set the default values for the various swing components used by
     * DateChooser.
     */
    protected void setDefaultStyle(Control component, int style) {
    }


    /**
     * Properties meant to contain user definitions.
     * 
     * @param component
     * @param style
     */
    protected void overrideStyle(Control component, int style) {
    }


    protected void setLocale(Locale locale) {
        this.locale = locale;
    }


    /**
     * Run the style definitions over the control. Calls overrideStyle which
     * allows overriding of the default style with the various style adapter
     * classes.
     * 
     * @param component
     * @param style
     */
    public void setStyle(Control component, int style) {
        setDefaultStyle(component, style);
        overrideStyle(component, style);
    }

}
