<?xml version="1.0" encoding="UTF-8"?>
<!-- /********************************************************* begin of 
	preamble ** ** Copyright (C) 2011 Software- und Organisations-Service GmbH. 
	** All rights reserved. ** ** This file may be used under the terms of either 
	the ** ** GNU General Public License version 2.0 (GPL) ** ** as published 
	by the Free Software Foundation ** http://www.gnu.org/licenses/gpl-2.0.txt 
	and appearing in the file ** LICENSE.GPL included in the packaging of this 
	file. ** ** or the ** ** Agreement for Purchase and Licensing ** ** as offered 
	by Software- und Organisations-Service GmbH ** in the respective terms of 
	supply that ship with this file. ** ** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT 
	HOLDERS AND CONTRIBUTORS "AS ** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, ** THE IMPLIED WARRANTIES OF MERCHANTABILITY 
	AND FITNESS FOR A PARTICULAR ** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
	THE COPYRIGHT OWNER OR CONTRIBUTORS ** BE LIABLE FOR ANY DIRECT, INDIRECT, 
	INCIDENTAL, SPECIAL, EXEMPLARY, OR ** CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
	NOT LIMITED TO, PROCUREMENT OF ** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
	DATA, OR PROFITS; OR BUSINESS ** INTERRUPTION) HOWEVER CAUSED AND ON ANY 
	THEORY OF LIABILITY, WHETHER IN ** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
	NEGLIGENCE OR OTHERWISE) ** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
	EVEN IF ADVISED OF THE ** POSSIBILITY OF SUCH DAMAGE. ********************************************************** 
	end of preamble*/ // -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="2.0">
	<xsl:output method="text" />
	<xsl:strip-space
		elements="description script log_mail_to log_mail_cc log_mail_bcc" />

	<!-- this parameter declares the output-folder of the generated single JS-object-files -->
	<xsl:param name="destination" select="'.'" />

	<!-- this parameter declares the character endcoding for the created object-files -->
	<!-- <xsl:param name="charencoding" select="'UTF-8'" /> -->
	<xsl:param name="charencoding" select="'iso-8859-1'" />

	<!-- modify destination (backslashes to shlashes, delete doublequotes, shlash 
		at the end) -->
	<xsl:param name="normalized_destination">
		<xsl:choose>
			<xsl:when test="$destination">
				<xsl:value-of
					select="concat(replace(translate($destination,'\&quot;','/'),'/$',''),'/')" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'./'" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:param>

	<!-- windows special for handling absolute destination paths -->
	<xsl:param name="file_destination">
		<xsl:choose>
			<xsl:when test="starts-with($normalized_destination,'file:')">
				<xsl:value-of select="$normalized_destination" />
			</xsl:when>
			<xsl:when test="contains($normalized_destination,':/')">
				<xsl:value-of select="concat('file:/',$normalized_destination)" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$normalized_destination" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:param>

	<!-- match some objects from a classic JobScheduler configuration file to 
		extract them as Hot Folder objects -->
	<xsl:template match="/spooler/config">
		<!--xsl:apply-templates mode="objects" select="params"/ --><!-- params have no attibutes to set the filename -->
		<xsl:apply-templates mode="objects"
			select="process_classes/process_class" />
		<xsl:apply-templates mode="objects" select="schedules/schedule" />
		<xsl:apply-templates mode="objects" select="locks/lock" />
		<xsl:apply-templates mode="objects" select="jobs/job" />
		<xsl:apply-templates mode="objects" select="job_chains/job_chain" />
		<xsl:apply-templates mode="objects"
			select="commands/order|commands/add_order" />
	</xsl:template>

	<!-- set filename for Hot Folder objects except orders and calls generator -->
	<xsl:template match="process_class|schedule|lock|job|job_chain"
		mode="objects">
		<xsl:param name="objname" select="name()" />
		<xsl:variable name="filename" select="concat(@name,'.',$objname,'.xml')" />
		<xsl:call-template name="stdout">
			<xsl:with-param name="objects" select="name(parent::*)" />
			<xsl:with-param name="filename" select="$filename" />
			<xsl:with-param name="position" select="position()" />
		</xsl:call-template>
		<xsl:apply-templates mode="write" select=".">
			<xsl:with-param name="filename" select="$filename" />
			<xsl:with-param name="objname" select="$objname" />
		</xsl:apply-templates>
	</xsl:template>

	<!-- set filename for Hot Folder orders and calls generator -->
	<xsl:template match="order|add_order" mode="objects">
		<xsl:param name="objname" select="'order'" />
		<xsl:variable name="filename"
			select="concat(@job_chain,',',@id,'.',$objname,'.xml')" />
		<xsl:call-template name="stdout">
			<xsl:with-param name="objects" select="'orders'" />
			<xsl:with-param name="filename" select="$filename" />
			<xsl:with-param name="position" select="position()" />
		</xsl:call-template>
		<xsl:apply-templates mode="write" select=".">
			<xsl:with-param name="filename" select="$filename" />
			<xsl:with-param name="objname" select="$objname" />
		</xsl:apply-templates>
	</xsl:template>

	<!-- generate Hot Folder objects -->
	<xsl:template match="*" mode="write">
		<xsl:param name="filename" />
		<xsl:param name="objname" select="name()" />
		<xsl:result-document
			href="{iri-to-uri(concat($file_destination,$filename))}" method="xml"
			encoding="{$charencoding}" output-version="1.0" indent="yes"
			cdata-section-elements="description script log_mail_to log_mail_cc log_mail_bcc">
			<xsl:element name="{$objname}">
				<xsl:apply-templates select="child::*|@*|comment()"
					mode="copy_node" />
			</xsl:element>
		</xsl:result-document>
	</xsl:template>

	<!-- copy attributes except name, id, job_chain and replace -->
	<xsl:template match="@*" mode="copy_node">
		<xsl:choose>
			<xsl:when test="name()='name'">
			</xsl:when>
			<xsl:when test="name()='id'">
			</xsl:when>
			<xsl:when test="name()='job_chain'">
			</xsl:when>
			<xsl:when test="name()='replace'">
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="." />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- copy comments with some cosmetics for a better look -->
	<xsl:template match="comment()" mode="copy_node">
		<xsl:text>&#xA;&#160;&#160;&#160;</xsl:text>
		<xsl:copy-of select="." />
		<xsl:if test="position()=last()">
			<xsl:text>&#xA;</xsl:text>
		</xsl:if>
	</xsl:template>

	<!-- copy elements -->
	<xsl:template match="*" mode="copy_node">
		<xsl:copy-of select="." />
	</xsl:template>

	<!-- only some lines to stdout -->
	<xsl:template name="stdout">
		<xsl:param name="objects" select="'objects'" />
		<xsl:param name="filename" />
		<xsl:param name="position" />

		<xsl:if test="$position=1">
			<xsl:message>
				&#xA;The following
				<xsl:value-of select="$objects" />
				will be generated in &quot;
				<xsl:value-of select="$normalized_destination" />
				&quot;:&#xA;
			</xsl:message>
		</xsl:if>
		<xsl:message>
			<xsl:value-of select="$position" />
			.&#x9;
			<xsl:value-of select="$filename" />&#xA;
		</xsl:message>
	</xsl:template>

</xsl:stylesheet>
