/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// ibm370.h
//                                                      (c) Joacim Zschimmer


#ifndef __IBM370_H
#define __IBM370_H

#ifndef __EMU370_H
//    #include <emu370h.h>
#endif

#if !defined __AREA_H
    #include "../kram/area.h"
#endif

#if !defined __DECIMAL_H
    #include "../kram/decimal.h"
#endif


namespace sos {

//-----------------------------------------------------------------------------

struct e370_Virt_addr;

//-----------------------------------------------------------------------------

struct Halbwort {
    Byte h [2];

    Halbwort ()  {}
    Halbwort (int i) {
        h [0] = (Byte) (i >> 8);
        h [1] = (Byte) i;
        //h [0] = *significant_byte_ptr (&i, 1);
        //h [1] = *significant_byte_ptr (&i, 0);
    }
    operator int2 () const  { return ((int2) h [0] << 8) | (int2) h [1]; }
    operator Const_area () const  { return Const_area( h, sizeof h ); }
};

struct Uhalbwort {
    Byte h [2];

    Uhalbwort ()  {}
    Uhalbwort (uint i) {
        h [0] = (Byte) (i >> 8);
        h [1] = (Byte) i;
        //h [0] = *significant_byte_ptr (&i, 1);
        //h [1] = *significant_byte_ptr (&i, 0);
    }
    operator uint2 () const  { return ((uint2) h [0] << 8) | (uint2) h [1]; }
    operator Const_area () const  { return Const_area( h, sizeof h ); }
};

struct Uwort3 {
    Byte w [3];

    Uwort3 ()  {}
    Uwort3 (uint4 i) {
        w [0] = *significant_byte_ptr (&i, 2);
        w [1] = *significant_byte_ptr (&i, 1);
        w [2] = *significant_byte_ptr (&i, 0);
    }
    operator uint4 () const {
        uint4 i;
        *significant_byte_ptr (&i, 2) = w [0];
        *significant_byte_ptr (&i, 1) = w [1];
        *significant_byte_ptr (&i, 0) = w [2];
        return i;
    }
    operator Const_area () const  { return Const_area( w, sizeof w ); }
};

struct Wort4
{
    Byte                        w [4];

                                Wort4                   ();
                                Wort4                   ( uint4 );
                                Wort4                   ( Halbwort );
                                Wort4                   ( Uhalbwort );
                                operator uint4          () const;
                                operator Const_area     () const;

    void                        object_store            ( std::ostream& );
    Wort4&                      object_load             ( std::istream& );

};

//istream& operator >> ( istream&, Wort4& );
//ostream& operator << ( ostream&, const Wort4& );

struct Doppelwort8 {
    operator Const_area () const  { return Const_area( b, sizeof b ); }
  private:
    Byte b [8];
};

typedef Wort4   Wort;

//-----------------------------------------------------------------------------
#if !defined SYSTEM_WIN         // Provisorisch
    typedef Wort4 Adr;
    typedef Wort4 Addr;
#else

struct Ibm370
{
    struct Addr : Wort4
    {
        Addr()  {}
        Addr( e370_Virt_addr );
        Addr( int4 );
        operator e370_Virt_addr() const;
    };
}; // Ibm370

typedef Ibm370::Addr Addr;
typedef Ibm370::Addr Adr;

#endif

} //namespace sos

#include "ibm370.inl"

#endif
