/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: thread.cxx 11394 2005-04-03 08:30:29Z jz $

#include "precomp.h"
#include "sos.h"
#include "thread_semaphore.h"
#include "thread.h"

#include <list>

namespace sos {

static int  tls_index;
static bool tls_index_set = false;

static std::list< Sos_ptr<Thread_container> >    container_list;
static Thread_semaphore                          list_lock ( "Thread_container" );


static struct Thread_static
{
    ~Thread_static() 
    {
        THREAD_LOCK( list_lock )  
        {
            if( tls_index_set )
            {
                container_list.clear();
         
                TlsFree( tls_index );
                tls_index_set = false;
            }
        }
    }
} z;

//---------------------------------------------------------------------------------thread_container

Thread_container* thread_container()
{
#   ifdef _WIN32

        if( !tls_index_set )
        {
            tls_index = TlsAlloc();
            if( tls_index == TLS_OUT_OF_INDEXES )  throw_mswin_error( "TlsAlloc", "thread_container" );
            tls_index_set = true;
        }

        Sos_ptr<Thread_container> tc = (Thread_container*)TlsGetValue( tls_index_set );
        if( !tc ) 
        {
            tc = SOS_NEW( Thread_container );
            TlsSetValue( tls_index, tc );
            THREAD_LOCK( list_lock )  container_list.push_back( tc );
        }

        return tc;

#    else

        static Thread_container tc;
        return &tc;

#   endif
}


//-------------------------------------------------------------------------------------------------

} //namespace sos
