/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: scheduler_object.h 14167 2010-12-30 15:11:58Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#ifndef __SCHEDULER_OBJECT_H
#define __SCHEDULER_OBJECT_H

#include "../javaproxy/com__sos__scheduler__engine__kernel__event__Event.h"
typedef javaproxy::com::sos::scheduler::engine::kernel::event::Event EventJ;

namespace sos {
namespace scheduler {

//---------------------------------------------------------------------------------Scheduler_object

struct Scheduler_object
{
    enum Type_code
    {
        type_none,

        type_active_schedulers_watchdog,
        type_cluster_member,
        type_database,
        type_database_order_detector,
        type_directory_file_order_source,
        type_directory_observer,
        type_directory_tree,
        type_exclusive_scheduler_watchdog,
        type_folder_subsystem,
        type_folder,
        type_subfolder_folder,
        type_heart_beat,
        type_heart_beat_watchdog_thread,
        type_http_file_directory,
        type_http_server,
        type_java_subsystem,
        type_job,
        type_job_chain,
        type_job_chain_folder,
        type_job_chain_group,
        type_job_chain_node,
        type_job_folder,
        type_job_subsystem,
        type_lock,
        type_lock_folder,
        type_lock_holder,
        type_lock_requestor,
        type_lock_subsystem,
        type_lock_use,
        type_web_service,
        type_web_service_operation,
        type_web_service_request,
        type_web_service_response,
        type_web_services,
        type_order,
        type_order_queue,
        type_order_subsystem,
        type_remote_configuration_observer,
        type_remote_scheduler,
        type_remote_schedulers_configuration,
        type_process,
        type_process_class,
        type_process_class_folder,
        type_process_class_subsystem,
        type_schedule,
        type_schedule_folder,
        type_schedule_subsystem,
        type_schedule_use,
        type_scheduler,
        type_scheduler_event_manager,
        type_scheduler_script,
        type_scheduler_script_folder,
        type_scheduler_script_subsystem,
        type_standing_order,
        type_standing_order_folder,
        type_standing_order_subsystem,
        type_subsystem_register,
        type_supervisor,
        type_supervisor_client,
        type_supervisor_client_connection,
        type_task,
        type_task_subsystem,
        type_xml_client_connection,
        type_scheduler_event2,
        type_event_subsystem,
      //type_subprocess_register
    };


    static string               name_of_type_code           ( Type_code );


                                Scheduler_object            ( Spooler*, IUnknown* me, Type_code );
    virtual                    ~Scheduler_object            ();


    Type_code                   scheduler_type_code         () const                                { return _scheduler_object_type_code; }
    void                    set_mail_xslt_stylesheet_path   ( const string& path )                  { _mail_xslt_stylesheet.release();  _mail_xslt_stylesheet_path = path; }
    Spooler*                    spooler                     () const                                { return _spooler; }

    virtual void                close                       ()                                      {}
    void                        report_event                (const EventJ&);
    //void                        report_event                ( const string& code );
    virtual ptr<Xslt_stylesheet> mail_xslt_stylesheet       ();
    virtual void                print_xml_child_elements_for_event( String_stream*, Scheduler_event* )  {}
    virtual string              obj_name                    () const                                { return name_of_type_code( _scheduler_object_type_code ); }
    virtual IDispatch*          idispatch                   ();
    IUnknown*                   iunknown                    () const                                { return _my_iunknown; }
    virtual void                write_element_attributes    ( const xml::Element_ptr& ) const;


    void                        complain_about_non_empty_attribute( const xml::Element_ptr&, const string& attribute_name );
    void                        assert_empty_attribute      ( const xml::Element_ptr&, const string& attribute_name );
    virtual void                self_check                  ()                                      {}

    Prefix_log*                 log                         ()                                      { return _log; }
    Database*                   db                          () const;
    Job_subsystem*              job_subsystem               () const;
    Task_subsystem*             task_subsystem              () const;
    Order_subsystem*            order_subsystem             () const;

    Spooler*                   _spooler;
    IUnknown*                  _my_iunknown;
    Type_code                  _scheduler_object_type_code;
    ptr<Xslt_stylesheet>       _mail_xslt_stylesheet;
    string                     _mail_xslt_stylesheet_path;
    ptr<Prefix_log>            _log;
};

//-------------------------------------------------------------------------------scheduler_object<>

//template<class T>
//struct scheduler_object : Scheduler_object, javabridge::has_proxy<T>
//{
//                                scheduler_object            (Spooler* s, IUnknown* me, Type_code tc) : Scheduler_object(s, me, tc) {}
//};

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

#endif
