/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
/**
 * \file spooler_module_script
 * \brief script-Verarbeitung ber JAVA interface
 * \details 
 * Verarbeitet Jobs, die in einer Scriptsprache geschrieben sind ber ein JAVA-Modul
 *
 * \author SS
 * \version 2.1.1 - 2010-05-07
 * \copyright  2010 by SS
 * <div class="sos_branding">
 *    <p> 2010 SOS GmbH - Berlin (<a style='color:silver' href='http://www.sos-berlin.com'>http://www.sos-berlin.com</a>)</p>
 * </div>
 */
#include "spooler.h"
#include "../javaproxy/java__lang__String.h"
#include "../zschimmer/java_com.h"

using namespace zschimmer::javabridge;

namespace sos {
namespace scheduler {

//-----------------------------------------------------------------Script_module::Script_module
    
Script_module::Script_module( Spooler* spooler, Prefix_log* log )
:
    Module( spooler, (File_based*)NULL, "::NO-INCLUDE-PATH::", log )
{
    _kind = kind_scripting_engine_java;
    _set = true;
}

//---------------------------------------------------Internal_instance_base::Script_module_instance
    
Script_module_instance::Script_module_instance( Module* module )             
: 
    Module_instance(module),
    _zero_(this+1),
    _java_module_instance( javaproxy::com::sos::scheduler::engine::kernel::scripting::APIModuleInstance::new_instance( module->_language, module->read_source_script()) )
{
}

//----------------------------------------------------------------Script_module_instance::add_obj
    
void Script_module_instance::add_obj( IDispatch* idispatch, const string& name )
{
    Z_LOG2("scheduler","Script_module_instance::add_obj name=" << name << "\n");

    ptr<javabridge::Java_idispatch> java_idispatch = Java_subsystem_interface::instance_of_scheduler_object(idispatch, name);
    _added_jobjects.push_back( java_idispatch );
    _java_module_instance.addObject( java_idispatch->get_jobject(), name );            // registriert das Object in ScriptInterface
}

//-----------------------------------------------------------Script_module_instance::call

Variant Script_module_instance::call( const string& name )
{
    Z_LOG2("scheduler","Script_module_instance::call name=" << name << "\n");
    Com_env env = _module->_java_vm->jni_env();
    javaproxy::java::lang::Object result = _java_module_instance.call(name);
    return (result == NULL) ? empty_variant : env.jobject_to_variant( result ); 
}

//-------------------------------------------------------------------Script_module_instance::call

Variant Script_module_instance::call( const string&, const Variant&, const Variant& )
{
    Z_LOG2("scheduler","Script_module_instance::call\n");
    assert(0), z::throw_xc( Z_FUNCTION );
}

//------------------------------------------------------------Script_module_instance::name_exists

bool Script_module_instance::name_exists( const string& name )
{
    Z_LOG2("scheduler","Script_module_instance::name_exists name=" << name << "\n");
    return (_java_module_instance.nameExists(name) != 0);  // weil name_exists jboolean zurck gibt (will JZ noch ndern)
}

//-------------------------------------------------------------------------------------------------

} //namespace scheduler
} //namespace sos

