/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: check_compiler.cxx 12891 2007-06-29 12:29:34Z jz $

#include "zschimmer.h"
#include "file.h"

#ifdef Z_WINDOWS
#   include <direct.h>      // getcwd()
#else
#   include <sys/utsname.h> // uname()
#endif

namespace zschimmer {

//-----------------------------------------------------------------------------------check_compiler

bool check_compiler( ostream* log )
{
    bool ok = true;


#   ifdef __hpux
        *log << "HP-UX  ";
#   endif

#   ifdef sparc
        *log << "Sparc  ";
#   endif

#   ifdef __IA64__
        *log << "ia64 ";
#   endif

#   ifdef Z_WINDOWS
    {
        *log << "Microsoft Windows ";

        OSVERSIONINFO v;  memset( &v, 0, sizeof v ); v.dwOSVersionInfoSize = sizeof v;
        GetVersionEx( &v );

        if( v.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS )  *log << "95/98/Me";
      //if( v.dwPlatformId == VER_PLATFORM_WIN32_NT      )  *log << "NT/2000/XP";
        *log << v.dwMajorVersion << '.' << v.dwMinorVersion;

        if( v.dwPlatformId == VER_PLATFORM_WIN32_NT )
        {
            switch( v.dwMajorVersion ) 
            {
                case 3:  *log << "  Windows NT 3.5.1";  break;
                case 4:  *log << "  Windows NT 4.0"  ;  break;

                case 5: switch( v.dwMinorVersion ) 
                        {
                            case 0:  *log << "  Windows 2000";                 break;
                            case 1:  *log << "  Windows XP";                   break;
                            case 2:  *log << "  Windows 2003 Server family";   break;
                            default: *log << "  Windows nach 2003";
                        }
                        break;

                case 6: switch( v.dwMinorVersion ) 
                        {
                            case 0:  *log << "  Windows Vista";                 break;
                        }
                        break;

                default:;
            }
        }

        if( v.szCSDVersion )  *log << ", " << v.szCSDVersion;

        *log << ", Build " << ( v.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS? v.dwBuildNumber & 0xFFFF : v.dwBuildNumber ) << '\n';

        *log << "GetCommandLine(): " << GetCommandLine() << '\n';
    }
#   else
    {
        struct utsname u;
        int ret = uname( &u );
        if( ret != -1 )
        {
            *log << "sys="       << u.sysname 
                 << " node="     << u.nodename 
                 << " release="  << u.release 
                 << " version="  << u.version 
                 << " machine="  << u.machine 
                 << "\n";
        }
    }
#   endif


#   ifdef __GNUC__
        *log << "Compiler GNU gcc " << __GNUC__ << '.' << __GNUC_MINOR__ << '.' << __GNUC_PATCHLEVEL__;
#   endif

#   ifdef _MSC_VER
        *log << "Compiler Microsoft Visual C++ " <<  ( _MSC_VER - 600 ) / 100 << '.' << ( _MSC_VER % 100 ) << "  (" <<_MSC_VER << ")";
#   endif

    *log << ", " __DATE__ << "\n";

    *log << "get_temp_path() => " << file::get_temp_path() << "\n";


    // Prfen, ob string(NULL,0) exeception "attempt to create string with null pointer" auslst.

    try
    {
//#     if _MSC_VER != 1400 || !defined( _DEBUG )  // Visual Studio 2005 bricht bei _DEBUG ab.
        // Microsoft Visual Studio 2005     (8.0.50727.42)  ndern: vc/include/xstring, Zeile 2013 auskommentieren: //_DEBUG_POINTER(_Ptr);
        // Microsoft Visual Studio 2005 SP1 (8.0.50727.762) ndern: vc/include/xstring, Zeile 2051 auskommentieren: //_DEBUG_POINTER(_Ptr);
        string a ( (const char*)NULL, (string::size_type)0 );
//#     endif
        //*log << "string(NULL,0) funktioniert!\n";
    }
    catch( const exception& x ) 
    { 
        *log << "Test: string(NULL,0) wirft Exception: " << x.what() << '\n';
        ok = false;
    }

    {
        char wd [ 1024+1 ];
        wd[0] = '\0';
        getcwd( wd, sizeof wd );
        *log << "working directory = " << wd << "\n";    
    }

    return ok;
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
