/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: embedded_files.cxx 13839 2010-04-06 09:53:57Z ss $

#include "zschimmer.h"
#include "embedded_files.h"

/**
* \change 2.0.224 - jira-XXX: Dynamisch eingebundes XSD verwenden
* \detail
*/
#include "file.h"
#include <time.h>


using namespace std;


namespace zschimmer {

//-------------------------------------------------------------------------------------------static

static Message_code_text error_codes[] =
{
    { "Z-NO-EMBEDDED-FILE", "Datei $1 ist nicht eingebettet" },
    {}
};

//-------------------------------------------------------------------------------------------Z_INIT

Z_INIT( embedded_files )
{
    add_message_code_texts( error_codes ); 
}

//--------------------------------------------------------------------------------get_embedded_file
    
const Embedded_file* Embedded_files::get_embedded_file( const string& filename ) const
{
    const Embedded_file* result = get_embedded_file_or_null( filename );
    if( !result )  throw_xc( "Z-NO-EMBEDDED-FILE", filename );
    return result;
}

//--------------------------------------------------------------------------------get_embedded_file
    
const Embedded_file* Embedded_files::get_embedded_file_or_null( const string& filename ) const
{
    
    for( const Embedded_file* f = _embedded_files; f->_filename; f++ )
    {
         if( f->_filename == filename )  return f;
    }

    return NULL;
}

//------------------------------------------------------------------------string_from_embedded_file

string Embedded_files::string_from_embedded_file( const string& filename ) const
{
    const Embedded_file* embedded_file = get_embedded_file( filename );
    return string( embedded_file->_content, embedded_file->_length );
}


/**
* \change 2.0.224 - JS-XXX: Liefert zu einem Key den Inhalt einer (Konfigurations-)datei.
* \detail
* Prioritt haben bei Schedulerstart dynamisch geladen Dateien. 
* Wenn es zu dem Schlssel kein dynamisch geladene Datei gibt, 
* dann wird das statische Dokument verwendet.
*/
string Embedded_and_dynamic_files::string_from_embedded_file(const string &filename)
{
	int size = _dynamic_files->size();
    // for( Embedded_file* f = _dynamic_files; f->_filename; f++ )
	for(int i = 0; i < size; ++i)
    {
		Dynamic_file* f = _dynamic_files->at(i);
		if(f != NULL) 
		{
			if( f->_filename == filename )  return string(f->_content->c_str(),f->_length);
		}
    }

	return _embedded_files.string_from_embedded_file(filename);
}


/**
* \change 2.0.224 - JS-XXX: File dynamisch laden
* \detail
* Ldt die Datei an dem angegebenen absoluten Pfad als string und macht den Inhalt fr das Programm verfgbar.
*/
void Embedded_and_dynamic_files::set_dynamic_file (string& filename, const string key )
{
	if(_dynamic_files == NULL)
		_dynamic_files = new  vector<Dynamic_file*>;
	Dynamic_file* dynamic_file = new Dynamic_file();
	string* content = new string(file::string_from_file(filename));
	dynamic_file->_content = content;
	dynamic_file->_filename = key;
	dynamic_file->_last_modified_time = time(NULL);
//	string* start = content;
//	while(*content++);
//	int size = content - start;
	dynamic_file->_length = content->size();

	_dynamic_files->push_back(dynamic_file);
}



//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
