/********************************************************* begin of preamble
**
** Copyright (C) 2003-2010 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: z_md5.cxx 13242 2008-01-01 20:36:00Z jz $        Joacim Zschimmer, Zschimmer GmbH, http://www.zschimmer.com

#include "zschimmer.h"
#include "z_md5.h"
#include "log.h"

namespace zschimmer {

using namespace std;
using namespace zschimmer::third_party::md5;

//-------------------------------------------------------------------------------------------------

static Message_code_text error_codes[] =
{
    { "Z-MD5-001", "MD5 sum does not consist of 32 hexadecimal digits: '$1'" },
    { "Z-MD5-002", "Invalid MD5 sum: $1" },
    { NULL }
};

//-------------------------------------------------------------------------------------------Z_INIT

Z_INIT( z_md5 )
{
    add_message_code_texts( error_codes ); 
}

//-------------------------------------------------------------------------------------Md5::set_hex

void Md5::set_hex( const string& hex_md5_sum )
{
    if( hex_md5_sum.length() != 2*md5_sum_byte_count )  z::throw_xc( "Z-MD5-001", hex_md5_sum );

    try
    {
        for( size_t i = 0; i < sizeof _bytes; i++ )
        {
            _bytes[ i ] = (Byte)( hex_to_digit( hex_md5_sum[ 2*i ] ) * 16 + hex_to_digit( hex_md5_sum[ 2*i+1 ] ) );
        }
    }
    catch( exception& x )  { throw_xc( "Z-MD5-002", x ); }
}

//----------------------------------------------------------------------------------------------md5

Md5 md5( const io::Byte_sequence& seq )
{
    Md5_calculator md5_sum_calculator;
    md5_sum_calculator.add( seq );
    return md5_sum_calculator.md5();
}

//-------------------------------------------------------------------Md5_calculator::Md5_calculator

Md5_calculator::Md5_calculator()
:
    _finished(false)
{
    MD5Init( &_md5_context );
}

//------------------------------------------------------------------------------Md5_calculator::add
    
void Md5_calculator::add( const io::Byte_sequence& seq )
{
    if( _finished )  throw_xc( Z_FUNCTION, "finished" );

    MD5Update( &_md5_context, seq.ptr(), seq.length() );
}

//------------------------------------------------------------------------------Md5_calculator::md5

Md5 Md5_calculator::md5()
{
    Md5 result;

    MD5Final( result._bytes, &_md5_context );
    _finished = true;

    return result;
}

//-------------------------------------------------------------------------------------------------

} //namespace zschimmer
